//tabs - https://developer.mozilla.org/en-US/Add-ons/Code_snippets/Tabbed_browser

Components.utils.import("resource://gre/modules/Downloads.jsm");

perConIkUserActivity = {

    isMonitoringEnabled: false,
    prefs: null, //nsIPrefBranch - https://developer.mozilla.org/en-US/docs/Mozilla/Tech/XPCOM/Reference/Interface/nsIPrefBranch
    svcUrls: {
        resolveTabEventUrl: function (eventType) {
            return this.tabEvent + eventType;
        },
        updateServices: function (port) {
            var port = perConIkUserActivity.prefs.getIntPref("port");

            this.saveEvent = new URI("http://localhost/web/save").port(port).toString();
            this.copyEvent = new URI("http://localhost/web/copy").port(port).toString();
            this.tabEvent = new URI("http://localhost/web/tab/").port(port).toString();
            this.bookmarkEvent = new URI("http://localhost/web/bookmark").port(port).toString();
            this.navigateEvent = new URI("http://localhost/web/navigate").port(port).toString();
        }
    },
    windowId: 0,
    appVersion: 0,
    appName: "Firefox",
    nextTabId: 1,
    downloadsList: null,
    bookmarkSvc: null,
    bookmarksObserver: {
        onItemAdded: function (id, folder, index, itemType, uri) {
            perConIkUserActivity.sendBookmarkUacaEvent(id, uri.spec);
        }
    },

    startup: function () {
        //settings observer - https://developer.mozilla.org/en/docs/Adding_preferences_to_an_extension
        this.prefs = Components.classes["@mozilla.org/preferences-service;1"]
        .getService(Components.interfaces.nsIPrefService)
        .getBranch("extensions.Gratex.PerConIK.UserActivity.");
        this.prefs.QueryInterface(Components.interfaces.nsIPrefBranch);
        this.prefs.addObserver("", this, false);

        this.isMonitoringEnabled = this.prefs.getBoolPref("enabled");
        this.svcUrls.updateServices();

        this.windowId = (new Date()).getTime();
        this.initAppVersion();

        this.updateIcon();

        //progress observer - https://developer.mozilla.org/en-US/Add-ons/Code_snippets/Progress_Listeners
        gBrowser.addProgressListener(this);

        //tab observer
        var tabContainer = gBrowser.tabContainer;
        tabContainer.addEventListener("TabOpen", this.tabAdded, false);
        tabContainer.addEventListener("TabClose", this.tabRemoved, false);
        tabContainer.addEventListener("TabSelect", this.tabSelected, false);


        //init tab ids and create open uaca events (tabAdded is not called on startup)
        for (var i = 0; i < gBrowser.tabs.length; ++i) {
            //            this.ensureTabId(gBrowser.tabs[i]);
            var tab = gBrowser.tabs[i];
            this.sendTabOpenUacaEvent(tab); //internally ensures tab id
        }

        //download listener
        Downloads.getList(Downloads.PUBLIC).then(function (list) {
            perConIkUserActivity.downloadsList = list;
            list.addView(perConIkUserActivity)
        });

        //bookmark observer
        this.bookmarkSvc = Cc["@mozilla.org/browser/nav-bookmarks-service;1"].getService(Ci.nsINavBookmarksService);
        this.bookmarkSvc.addObserver(this.bookmarksObserver, false);
    },

    shutdown: function () {
        this.bookmarkSvc.removeObserver(this.bookmarksObserver);

        this.downloadsList.removeView(this);

        var tabContainer = gBrowser.tabContainer;
        tabContainer.removeEventListener("TabSelect", this.tabSelected, false);
        tabContainer.removeEventListener("TabClose", this.tabRemoved, false);
        tabContainer.removeEventListener("TabOpen", this.tabAdded, false);

        gBrowser.removeProgressListener(this);

        this.prefs.removeObserver("", this);
    },

    //options observer
    observe: function (subject, topic, data) {
        if (topic != "nsPref:changed") {
            return;
        }

        if (data == "port") {
            this.svcUrls.updateServices();
        } else if (data == "enabled") {
            this.isMonitoringEnabled = this.prefs.getBoolPref("enabled");
            this.updateIcon();
        }
    },


    onLocationChange: function (progress, request, uri, flags) {
        if (!(flags & Ci.nsIWebProgressListener.LOCATION_CHANGE_SAME_DOCUMENT)) { // https://developer.mozilla.org/en-US/Add-ons/Code_snippets/Progress_Listeners

            if (request) {//used to filter out switching to another tab. In https://developer.mozilla.org/en-US/docs/Mozilla/Tech/XPCOM/Reference/Interface/nsIWebProgressListener#onLocationChange() are not defined cases when it is null but empirically it looks that it works :/. And I don't know better way to filter out tab switches.
                //  alert(uri.spec);
                //    window.Application.console.log("lc>" + uri.spec);

                //window.Application.console.log(getActiveTab);

                this.addUacaEvent({
                    tabId: this.ensureTabId(gBrowser.selectedTab),
                    url: uri.spec,
                    transitionTypeUri: "http://perconik.gratex.com/useractivity/enum/webnavigateevent/transitiontype#unknown"
                }, this.svcUrls.navigateEvent);
            }
        }
    },

    onStateChange: function () { },
    onProgressChange: function () { },
    onStatusChange: function () { },
    onSecurityChange: function () { },

    //don't use 'this'
    tabSelected: function (event) {
        perConIkUserActivity.sendTabSwitchUacaEvent(event.target);
    },

    //don't use 'this'
    tabAdded: function (event) {
        perConIkUserActivity.sendTabOpenUacaEvent(event.target);
    },

    //don't use 'this'
    tabRemoved: function (event) {
        perConIkUserActivity.sendTabCloseUacaEvent(event.target);
    },

    //don't use 'this'
    onCopy: function (e) {
        perConIkUserActivity.addUacaEvent({
            tabId: perConIkUserActivity.ensureTabId(gBrowser.selectedTab),
            url: gBrowser.selectedBrowser.currentURI.spec,
            content: window.content.getSelection().toString()
        }, perConIkUserActivity.svcUrls.copyEvent);
    },

    onDownloadAdded: function (download) {
        if (!download.perConIkUserActivity_Handled) { //not handled in another Firefox window (not necessary the correct one todo:fix)
            download.perConIkUserActivity_Handled = true;
            this.addUacaEvent({
                url: download.source.url,
                name: download.target.path
            }, this.svcUrls.saveEvent);
        }
    },

    onToolbarIconClick: function () {

        //try
        //{
        //    var dd = Downloads.getList(Downloads.ALL);
        //    alert(dd);
        //}
        //catch (err) {
        //    alert("a");
        //}
        //alert("c");
        // alert(JSON.stringify({ a: "aa", b: 22 }));
        this.isMonitoringEnabled = !this.isMonitoringEnabled;
        this.prefs.setBoolPref("enabled", this.isMonitoringEnabled);
        this.updateIcon();

        //alert(this.err);

        //   alert(this.ensureTabId(gBrowser.selectedTab));
        //alert(gBrowser.selectedTab.perConIkUaId);
    },

    sendTabOpenUacaEvent: function (tab) {
        var tabId = this.ensureTabId(tab);
        var tabBrowser = gBrowser.getBrowserForTab(tab);

        this.addUacaEvent({
            tabId: tabId,
            url: tabBrowser.currentURI.spec
        }, this.svcUrls.resolveTabEventUrl("open"));
    },
    sendTabCloseUacaEvent: function (tab) {
        var tabId = this.ensureTabId(tab);
        var tabBrowser = gBrowser.getBrowserForTab(tab);

        this.addUacaEvent({
            tabId: tabId,
            url: tabBrowser.currentURI.spec
        }, this.svcUrls.resolveTabEventUrl("close"));
    },
    sendTabSwitchUacaEvent: function (tab) {
        var tabId = this.ensureTabId(tab);
        var tabBrowser = gBrowser.getBrowserForTab(tab);

        this.addUacaEvent({
            tabId: tabId,
            url: tabBrowser.currentURI.spec
        }, this.svcUrls.resolveTabEventUrl("switchto"));
    },
    sendSaveUacaEvent: function (name, url) {
        this.addUacaEvent({
            url: url,
            name: name
        }, this.svcUrls.saveEvent);
    },
    sendBookmarkUacaEvent: function (id, url) {
        this.addUacaEvent({
            url: url,
            name: this.bookmarkSvc.getItemTitle(id)
        }, this.svcUrls.bookmarkEvent);
    },

    ensureTabId: function (tab) {
        if (tab.perConIkUaId) {
            return tab.perConIkUaId;
        }
        tab.perConIkUaId = this.nextTabId++;
        return tab.perConIkUaId;
    },

    updateIcon: function () {
        if (document.getElementById("perconik-ua-toolbar-button") != null) {
            document.getElementById("perconik-ua-toolbar-button").className = this.isMonitoringEnabled ? "perconik-ua-icon-active" : "perconik-ua-icon-inactive";
        }
    },

    initAppVersion: function () {
        //http://stackoverflow.com/a/4967363
        var appInfo = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo);
        this.appVersion = appInfo.version;
    },

    addUacaEvent: function (event, srvUrl) {
        if (this.isMonitoringEnabled === true) {
            event.sessionId = this.windowId;
            event.appName = this.appName;
            event.appVersion = this.appVersion;
            event.timestamp = this.xsdUtcDateTime();

            var xhr = new XMLHttpRequest();
            xhr.open("POST", srvUrl, true);
            xhr.setRequestHeader("Content-Type", "application/json; charset=utf-8");
            xhr.send(JSON.stringify(event));
        }
    },

    xsdUtcDateTime: function () {
        var dNow = new Date();
        return dNow.toISOString();
    }
}


window.addEventListener("load", function (e) { perConIkUserActivity.startup(); }, false);
window.addEventListener("unload", function (e) { perConIkUserActivity.shutdown(); }, false);

document.addEventListener('copy', perConIkUserActivity.onCopy, true);


